@@ -1,6 +1,7 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2001-2004 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+   Copyright (C) 2001-2005 Hewlett-Packard Co
+   Copyright (C) 2007 David Mosberger-Tang
+	Contributed by David Mosberger-Tang <dmosberger@gmail.com>
 
 This file is part of libunwind.
 
@@ -23,52 +24,12 @@ LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
-#include <alloca.h>
-#include <stdlib.h>
-
 #include "unwind_i.h"
 
 #ifdef HAVE_SYS_UC_ACCESS_H
 # include <sys/uc_access.h>
 #endif
 
-#if UNW_DEBUG
-
-HIDDEN const char *
-ia64_strloc (ia64_loc_t loc)
-{
-  static char buf[128];
-
-  if (IA64_IS_NULL_LOC (loc))
-    return "<null>";
-
-  buf[0] = '\0';
-
-  if (IA64_IS_MEMSTK_NAT (loc))
-    strcat (buf, "memstk_nat(");
-  if (IA64_IS_UC_LOC (loc))
-    strcat (buf, "uc(");
-  if (IA64_IS_FP_LOC (loc))
-    strcat (buf, "fp(");
-
-  if (IA64_IS_REG_LOC (loc))
-    sprintf (buf + strlen (buf), "%s", unw_regname (IA64_GET_REG (loc)));
-  else
-    sprintf (buf + strlen (buf), "0x%llx",
-	     (unsigned long long) IA64_GET_ADDR (loc));
-
-  if (IA64_IS_FP_LOC (loc))
-    strcat (buf, ")");
-  if (IA64_IS_UC_LOC (loc))
-    strcat (buf, ")");
-  if (IA64_IS_MEMSTK_NAT (loc))
-    strcat (buf, ")");
-
-  return buf;
-}
-
-#endif /* UNW_DEBUG */
-
 #ifdef UNW_REMOTE_ONLY
 
 /* unw_local_addr_space is a NULL pointer in this case.  */
@@ -387,7 +348,7 @@ get_static_proc_name (unw_addr_space_t as, unw_word_t ip,
 		      char *buf, size_t buf_len, unw_word_t *offp,
 		      void *arg)
 {
-  return _Uelf64_get_proc_name (getpid (), ip, buf, buf_len, offp);
+  return _Uelf64_get_proc_name (as, getpid (), ip, buf, buf_len, offp);
 }
 
 HIDDEN void
