@@ -1,5 +1,5 @@
 /* libunwind - a platform-independent unwind library
-   Copyright (C) 2003-2004 Hewlett-Packard Co
+   Copyright (C) 2003-2005 Hewlett-Packard Co
 	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
@@ -26,19 +26,20 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #ifndef UNW_REMOTE_ONLY
 
 #include <dlfcn.h>
+#include <string.h>
 #include <unistd.h>
 
-#include "internal.h"
-#include "tdep.h"
+#include "libunwind_i.h"
 
 #include "elf64.h"
 
 HIDDEN int
 tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
-		    unsigned long *segbase, unsigned long *mapoff)
+		    unsigned long *segbase, unsigned long *mapoff,
+		    char *path, size_t pathlen)
 {
   struct load_module_desc lmd;
-  const char *path;
+  const char *path2;
 
   if (pid != getpid ())
     {
@@ -52,10 +53,16 @@ tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
   *segbase = lmd.text_base;
   *mapoff = 0;			/* XXX fix me? */
 
-  path = dlgetname (&lmd, sizeof (lmd), NULL, 0, 0);
-  if (!path)
+  path2 = dlgetname (&lmd, sizeof (lmd), NULL, 0, 0);
+  if (!path2)
     return -UNW_ENOINFO;
-
+  if (path)
+    {
+      strncpy(path, path2, pathlen);
+      path[pathlen - 1] = '\0';
+      if (strcmp(path, path2) != 0)
+        Debug(1, "buffer size (%d) not big enough to hold path\n", pathlen);
+    }
   Debug(1, "segbase=%lx, mapoff=%lx, path=%s\n", *segbase, *mapoff, path);
 
   return elf_map_image (ei, path);
